/*
 * disol.c
 *       --- Disassemble Outline Data
 *
 *  Copyright (C) 1993  by Hirotsugu KAKUGAWA
 *  31 Dec 1993  
 *  20 Jan 1994  New output format
 *
 */ 



#include <stdio.h>
#include <stdlib.h>
#include <varargs.h>
#include "VF.h"
/*#include "_VF.h"*/

#define CommentLine printf

int
main(argc, argv)
  int  argc;
  char **argv;
{
  int   Fd, i;
  int   CharCode, CCode, Ch, Coord;
  char  *FontName, *Vfcap, *CoordStr;
  long  *VFData;

  FontName = "min";
  Vfcap    = NULL;
  Coord    = 0;

  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch (argv[i][1]){
      case 'f':
	FontName = argv[++i];
	break;
      case 'v':
	Vfcap = argv[++i];
	break;
      case 'c':
	Coord = 1;
	break;
      case 'h':
      default:
	printf("disol  --- disassemble outline data\n");
	printf("Usage disol [options] [JISCODE1 JISCODE2 ....]\n");
	printf("options: \n");
	printf(" -f FONT_ENTRY  : specify font entry name. (`min' is default)\n");
	printf(" -v VFONTCAP    : set vfontcap path.\n");	
      /*printf(" -c             : choose font original coordinates. %s\n");*/
	exit(0);
      }
    } else {
      Ch = i;
      break;
    }
  }

  /* Init VFlib */
  if (VF_Init(Vfcap) <0){
    fprintf(stderr, "VFlib init error\n");    
    exit(1);
  }

  /* OPEN THE FONT */
  if ((Fd = VF_OpenFont(FontName)) < 0){
    fprintf(stderr, "open error; %s\n", FontName);
    exit(-1);
  }

  printf(";; OUTLINES OF FONT ENTRY %s\n\n", FontName);
  
  while (Ch < argc){
    sscanf(argv[Ch], "%x", &CharCode);
    Ch++;

    /* GET VECTOR FONT DATA */
    VFData = VF_GetOutline(CharCode, Fd);
    if (VFData == NULL){
      printf(";; CAN'T GET OUTLINE FOR CHARACTOR $%x of Font %s\n", 
	     CharCode, FontName);
      continue;
    }

    /* DISASSEMBLE IT */
    DisVFData(VFData, Fd);

    /* RELEASE OUTLINE */
    VF_FreeOutline(VFData, Fd);
  }

  printf("END\n");

  /* CLOSE THE FONT */
  VF_CloseFont(Fd);

  /* THEN, FINISH USING VFLIB */
  VF_Deinit();
}




/*
 * Disassemble Vector Font Data returned by VF_GetOutline()
 */ 

DisVFData(vfdata, fd)
  long  *vfdata;
  int   fd;
{
  long          cmd, *ptr;
  unsigned int  x, y;
  int           m;

  if (vfdata == NULL)
    return;
  PrintCCode(vfdata[0]);
  ptr = &vfdata[2];
  do {
    m = 0;
    cmd = *ptr;
    ptr++; 
    if (cmd == 0L){
      PrintEnd();
      EmptyLine();
    } else if ((cmd & VFD_TOKEN) == VFD_TOKEN){
      PrintToken();
      if ((cmd & VFD_CHAR) == VFD_CHAR)
	Print(&m, "CH");
      if ((cmd & VFD_CWCURV) == VFD_CWCURV)
	Print(&m, "C1");
      if ((cmd & VFD_CCWCURV) == VFD_CCWCURV)
	Print(&m, "C2");
      if ((cmd & VFD_LINE) == VFD_LINE)
	Print(&m, "LI");
      if ((cmd & VFD_ARC) == VFD_ARC)
	Print(&m, "AR");
      if ((cmd & VFD_BEZ) == VFD_BEZ)
	Print(&m, "BE");
      Newline();
    } else {
      x = VFD_GET_X(cmd);
      y = VFD_GET_Y(cmd);
      PrintXY(x, y);
    }
  } while (cmd != 0L);
}


EmptyLine()
{
  printf("\n");
}

PrintCCode(n)
  int n;
{
  printf("CHAR    ");
  printf("$%x\n", n);
}

PrintToken()
{
  printf("        ");
  printf("TOKEN   ");
}

Print(mp, str)
  int  *mp;
  char *str;
{
  if (*mp == 0)
    printf("[");
  if (*mp > 0)
    printf(",");
  printf("%s", str);
  (*mp)++;
}
Newline()
{
  printf("]\n");
}

PrintEnd()
{
  printf("        ");
  printf("END     \n");
}

PrintXY(x, y)
  int x, y;
{
  printf("        ");
  printf("XY      ");
  printf("%d %d\n",x,y);
}

